#! /usr/bin/env node

var esprima = require("esprima");
var fs = require("fs");
var colors = require("./colors.json");

if(!process.argv[2]){
  echoUsage();
}

process.argv.slice(2).forEach(function (path) {

  var code = fs.readFileSync(path, "utf-8");
  var ast = esprima.parse(removeShebang(code), {
    tokens: true,
    range: true,
    comment: true
  });
  var colorStartIndexes = [];
  var colorEndIndexes = [];

  ast.tokens.concat(ast.comments).forEach(function (token) {
    var range = token.range;
    colorStartIndexes[range[0]] = isOperator(token) ? "Operator" : token.type;
    colorEndIndexes[range[1]] = true;
  });

  [].forEach.call(code, function (char, index) {
    if (colorEndIndexes[index]) {
      endColor();
    }
    var type = colorStartIndexes[index];
    if (type && colors[type]) {
      startColor(colors[type]);
    }
    printf(char);
  })
})

function isOperator(token) {
  return token.type === "Punctuator" && "()[]{},.:;".indexOf(token.value) === -1;
}

function removeShebang(code) {
  return code.replace(/^#!/, "//");
}

function startColor(color) {
  var ansiColors = {
    "normal": 39,
    "black": 30,
    "red": 31,
    "green": 32,
    "yellow": 33,
    "blue": 34,
    "magenta": 35,
    "cyan": 36,
    "white": 37,
    "brightBlack": 90,
    "brightRed": 91,
    "brightGreen": 92,
    "brightYellow": 93,
    "brightBlue": 94,
    "brightMagenta": 95,
    "brightCyan": 96,
    "brightWhite": 97
  }
  printf("\x1b[" + ansiColors[color] + "m");
}

function endColor() {
  printf("\x1b[0m");
}

function echoUsage(){
  console.log("Usage: catjs [file ...]");
  process.exit();
}

function printf(str) {
  process.stdout.write(str);
}